#!/usr/bin/perl
#
# list_countries.pl
#
# version 1.00, 11-28-03
#
# Copyright 2003, Michael Robinton <michael@bizsystems.com>
   
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
   
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
   
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

use lib qw(blib/lib);
use Net::DNSBL::Utilities qw(list_countries);

my($countries,$code3s,$names) = list_countries;
foreach (0..$#{$countries}) {
  next unless $countries->[$_];
  print "$countries->[$_],\t$code3s->[$_],\t$names->[$_]\n";
}
 